package dtos;

import io.zipcoder.tc_spring_poll_application.exception.ResourceNotFoundException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.MessageSource;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@ControllerAdvice
public class RestExceptionHandler {

    MessageSource messageSource;

    RestExceptionHandler(@Autowired MessageSource messageSource){
        this.messageSource = messageSource;
    }

    @ExceptionHandler(ResourceNotFoundException.class)
    public ResponseEntity<?> handleResourceNotFoundException(ResourceNotFoundException rnfe, HttpServletRequest request){
        ErrorDetail detail = new ErrorDetail();
        detail.setTitle(rnfe.getMessage());
        detail.setStatus(rnfe.hashCode());
        detail.setDetail(rnfe.getMessage());
        detail.setDeveloperMessage(rnfe.getLocalizedMessage());
        detail.setTimeStamp(new Date().getTime());
        return new ResponseEntity<>(detail, HttpStatus.NOT_FOUND);
    }

    @ExceptionHandler(MethodArgumentNotValidException.class)
    public ResponseEntity<?> handleValidationError(MethodArgumentNotValidException manve, HttpServletRequest request){
        ErrorDetail detail = new ErrorDetail();

        List<FieldError> fieldErrors = manve.getBindingResult().getFieldErrors();

        for (FieldError f: fieldErrors) {

            List<ValidationError> validationErrorList = detail.getErrors().get(f.getField());
            if (validationErrorList == null) {
                validationErrorList = new ArrayList<>();
                detail.getErrors().put(f.getField(), validationErrorList);
            }
            ValidationError validationError = new ValidationError();
            validationError.setCode(f.getCode());
            validationError.setMessage(messageSource.getMessage(f, null));
            validationErrorList.add(validationError);
        }
        return new ResponseEntity<>(detail, HttpStatus.BAD_REQUEST);
    }
}
